local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local x = s.UpdatedVariables

a.BuffPreSteadyFocus = false
a.WatcherStarted = false
a.Wearing2T13 = false

a.ResetSteadyFocus = {
	53209--[[Chimera Shot]],
	19434--[[Aimed Shot]],
	5116--[[Concussive Shot]],
	20736--[[Distracting Shot]],
	53351--[[Kill Shot]],
	2643--[[Multi-Shot]],
	1978--[[Serpent Sting]],
	34490--[[Silencing Shot]],
	19801--[[Tranquilizing Shot]],
	82654--[[Widow Venom]],
	19386--[[Wyvern Sting]],
	109248--[[Binding Shot]],
}

a.PreSteadyFocus = function()
	if a.BuffPreSteadyFocus then
		return not s.Casting(56641--[[Steady Shot]], "player")
	else
		return s.Casting(56641--[[Steady Shot]], "player")
	end
end

a.stopPreSteadyFocus = function()
	if a.WatcherStarted then
		s.UnregisterModuleEvent(AddonName, "UNIT_SPELLCAST_SUCCEEDED")
		s.UnregisterModuleEvent(AddonName, "COMBAT_LOG_EVENT_UNFILTERED")
		a.WatcherStarted = false
		a.BuffPreSteadyFocus = false
		a.print(L["Not specced into Marksman, stopped watching for pre Steady Focus."])
	end
end

a.startPreSteadyFocus = function()
	if not a.WatcherStarted then
		s.RegisterModuleEvent(AddonName, "UNIT_SPELLCAST_SUCCEEDED", function (event, ...)
			local unit, _, _, _, spellID = select(1, ...)
			if unit == "player" then
				if spellID == 56641--[[Steady Shot]] then
					a.BuffPreSteadyFocus = true
				elseif  tContains(a.ResetSteadyFocus, spellID)  then
					a.BuffPreSteadyFocus = false
				end
			end
		end)
		s.RegisterModuleEvent(AddonName, "COMBAT_LOG_EVENT_UNFILTERED", function (event, ...)
			local _, event, _, _, sourceName, _, _, _, _, _, _, _, spellName = select(1, ...)
			if (event == "SPELL_AURA_APPLIED" or event == "SPELL_AURA_REFRESH") and sourceName == UnitName("player") then
				if spellName == s.SpellName(53224--[[Steady Focus]]) then
					a.BuffPreSteadyFocus = false
				end
			end
		end)
		a.WatcherStarted = true
		a.print(L["Marksman detected, watching for pre Steady Focus."])
	end
end

if s.Spec() == 2 and not a.WatcherStarted then
	a.startPreSteadyFocus()
end

s.RegisterModuleEvent(AddonName, "ACTIVE_TALENT_GROUP_CHANGED", function (event, ...)
	if s.Spec() == 2 and not a.WatcherStarted then
		a.startPreSteadyFocus()
	else
		a.stopPreSteadyFocus()
	end
end)

a.HunterT13 = {
	HeadSlot = {78793, 77030, 78698}, --[[Head]]
	ShoulderSlot = {78832, 77032, 78737},  --[[Shoulder]]
	ChestSlot = {78756, 77028, 78661},  --[[Chest]]
	LegsSlot = {78804, 77031, 78709}, --[[Legs]]
	HandsSlot = {78769, 77029, 78674}, --[[Gloves]]
}

a.scanFor2T13 = function()
	local count = 0
	for slot, list in pairs(a.HunterT13) do
		if s.Equipped(list, slot) then
			count = count + 1
		end
	end
	if count >= 2 then
		if not a.Wearing2T13 then
			a.print(L["2T13 set bonus equipped."])
		end
		a.Wearing2T13 = true
	else
		if a.Wearing2T13 then
			a.print(L["No longer wearing 2T13."])
		end
		a.Wearing2T13 = false
	end
end

s.RegisterModuleEvent(AddonName, "PLAYER_EQUIPMENT_CHANGED", a.scanFor2T13)
a.scanFor2T13()

a.HasteCDs = {
	49016--[[Unholy Frenzy]],
	80353--[[Time Warp]],
	2825--[[Bloodlust]],
	32182--[[Heroism]],
	90355--[[Ancient Hysteria]],
	26297--[[Berserking]],
	3045--[[Rapid Fire]],
	105919--[[Chronohunter]],
}

-- Determines if you will have enough Focus to cast
-- "targetSpell" when it comes off CD if you cast
-- "spellInQuestion" right now.
a.canStillCast = function(targetSpell, spellInQuestion)
	local timeTillTarget, _ = s.SpellCooldown(targetSpell)
	local _, gcd = s.GlobalCooldown()
	local regenRate = GetPowerRegen()
	local regenShot = 0
	if s.Casting(77767--[[Cobra Shot]], "player") then
		regenShot = 14
	end
	if s.Casting(56641--[[Steady Shot]], "player") then
		regenShot = 14
	end
	if a.Wearing2T13 then
		regenShot = regenShot * 2
	end
	if s.Buff(53224--[[Steady Focus]], "player") and regenShot > 0 then
		regenShot = regenShot + 3
	end
	regenRate = format("%.2f", regenRate)
	if timeTillTarget < gcd then
		return false
	end
	if (s.Power("player", SPELL_POWER_FOCUS) + timeTillTarget * regenRate) < (s.SpellCost(targetSpell, SPELL_POWER_FOCUS) + s.SpellCost(spellInQuestion, SPELL_POWER_FOCUS) - regenShot) then
		return false
	end
	return true
end

a.spells["Auto Shot"] = {
	ID = 75--[[Auto Shot]],
	EnemyTargetNeeded = true,
	NotIfActive = true,
	Check = function(z)
		z.FlashColor = s.If(x.ActiveEnemy, "Green", "Pink")
		return x.NoCC and not s.Shooting()
	end,
	NoGCD = true,
}

a.spells["Pet Attack"] = {
	EnemyTargetNeeded = true,
	Override = function(z)
		if x.NoCC and x.PetAlive and x.PetActiveEnemy and not s.SameTargetAsPet() then
			s.FlashPet(PET_ACTION_ATTACK)
		end
	end,
	FlashColor = "Pink",
	NoGCD = true,
}

a.spells["Pet Assist"] = {
	Override = function(z)
		if x.ActiveEnemy and x.PetAlive and not x.PetActiveEnemy and s.Shooting() then
			s.FlashPet(PET_MODE_ASSIST)
		end
	end,
	FlashColor = "Pink",
	NoGCD = true,
}

a.spells["Hunter's Mark"] = {
	ID = 1130--[[Hunter's Mark]],
	Debuff = 1130--[[Hunter's Mark]],
	EarlyRefresh = 5,
	Check = function(z)
		return x.ShouldTemporaryBuff and not s.HasGlyph(132106--[[Glyph of Marked for Death]])
	end,
	EnemyTargetNeeded = true,
}

a.spells["Aspect of the Hawk"] = {
	Type = "form",
	ID = 13165--[[Aspect of the Hawk]],
	Buff = {
		13165--[[Aspect of the Hawk]],
		109260--[[Aspect of the Iron Hawk]],
	},
	BuffUnit = "player",
	NotWhileMoving = true,
	NoGCD = true,
	Check = function(z)
		return x.ShouldPermanentBuff and not s.Moving("player")
	end,
	FlashID = {
		13165--[[Aspect of the Hawk]],
		109260--[[Aspect of the Iron Hawk]],
	},
}

a.spells["Aspect of the Fox while moving"] = {
	Type = "form",
	ID = 82661--[[Aspect of the Fox]],
	Buff = 82661--[[Aspect of the Fox]],
	BuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		return x.ShouldPermanentBuff and s.Moving("player")
	end,
}

a.spells["Explosive Trap (AoE)"] = {
	ID = 13813--[[Explosive Trap]],
	Check = function(z)
		return x.ActiveEnemy
	end,
	Run = function(z)
		if s.Buff(82921--[[Bombardment]], "player") then
			z.FlashColor = "White"
			z.Continue = false
		else
			z.FlashColor = "Purple"
			z.Continue = true
		end
	end,
	NoRangeCheck = true,
}

a.spells["Focus Fire at 5 stacks"] = {
	ID = 82692--[[Focus Fire]],
	RequireBuff = 19623--[[Frenzy]],
	RequireStack = 5,
	RequireBuffUnit = "pet",
	Check = function(z)
		return x.ShouldTemporaryBuff and x.ActiveEnemy and x.PetAlive 
	end,
	NoGCD = true,
}

a.spells["Serpent Sting if not ticking"] = {
	ID = 1978--[[Serpent Sting]],
	MyDebuff = 1978--[[Serpent Sting]],
	Run = function(z)
		local travelTime = s.LastSpellTravelTime(77767--[[Cobra Shot]]) or 0
		z.EarlyRefresh = s.CastTime(77767--[[Cobra Shot]]) + travelTime
	end,
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Serpent Sting if not ticking and target HP <= 90%"] = {
	ID = 1978--[[Serpent Sting]],
	MyDebuff = 1978--[[Serpent Sting]],
	CheckLast = function(z)
		return x.NoCC and s.HealthPercent("target") <= 90
	end,
	EnemyTargetNeeded = true,
}

a.spells["Serpent Sting if not ticking and TTD  >= 10"] = {
	ID = 1978--[[Serpent Sting]],
	MyDebuff = 1978--[[Serpent Sting]],
	Run = function(z)
		local travelTime = s.LastSpellTravelTime(77767--[[Cobra Shot]]) or 0
		z.EarlyRefresh = s.CastTime(77767--[[Cobra Shot]]) + travelTime
		if s.TimeToDie() >= 10 then
			z.Continue = false
			z.FlashColor = "White"
		else
			z.Continue = true
			z.FlashColor = "Pink"
		end
	end,
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Chimera Shot if target HP <= 90%"] = {
	ID = 53209--[[Chimera Shot]],
	CheckLast = function(z)
		return x.NoCC and s.HealthPercent("target") <= 90
	end,
	EnemyTargetNeeded = true,
}

a.spells["Bestial Wrath if focus > 60"] = {
	ID = 19574--[[Bestial Wrath]],
	Check = function(z)
		return x.ShouldTemporaryBuff and s.InCombat() and s.Power("player", SPELL_POWER_FOCUS) > 60 
	end,
}

a.spells["Multi-Shot (AoE)"] = {
	ID = 2643--[[Multi-Shot]],
	Check = function(z)
		return x.NoCC
	end,
	Run = function(z)
		if s.Buff(82921--[[Bombardment]], "player") then
			z.FlashColor = "White"
			z.Continue = false
		else
			z.FlashColor = "Purple"
			z.Continue = true
		end
	end,
	EnemyTargetNeeded = true,
}

a.spells["Steady Shot (AoE)"] = {
	ID = 56641--[[Steady Shot]],
	RequireBuff = 82921--[[Bombardment]],
	RequireBuffUnit = "player",
	Check = function(z)
		return x.NoCC 
	end,
	FlashColor = "White",
	EnemyTargetNeeded = true,
}

a.spells["Cobra Shot (AoE)"] = {
	ID = 77767--[[Cobra Shot]],
	Check = function(z)
		local _, time = s.LastSpellTravelTime(2643--[[Multi-Shot]])
		if not time then
			return false
		else
			return x.NoCC and (GetTime() - time ) < 2
		end
	end,
	FlashColor = "Purple",
	EnemyTargetNeeded = true,
	Continue = true,
}

a.spells["Kill Shot"] = {
	ID = 53351--[[Kill Shot]],
	Check = function(z)
		return x.NoCC and not a.PreSteadyFocus()
	end,
	EnemyTargetNeeded = true,
}

a.spells["A Murder of Crows"] = {
	ID = 120657--[[A Murder of Crows]],
	Check = function(z)
		return x.NoCC and s.HasTalent(120657--[[A Murder of Crows]]) and not a.PreSteadyFocus()
	end,
	EnemyTargetNeeded = true,
}

a.spells["Stampede"] = {
	ID = 121818--[[Stampede]],
	Check = function(z)
		return x.NoCC 
	end,
	EnemyTargetNeeded = true,
	Continue = true,
	FlashColor = "Yellow",
}

a.spells["Glaive Toss"] = {
	ID = 117050--[[Glaive Toss]],
	Check = function(z)
		return x.NoCC and s.HasTalent(117050--[[Glaive Toss]])
	end,
	EnemyTargetNeeded = true,
}

a.spells["Dire Beast"] = {
	ID = 120679--[[Dire Beast]],
	Check = function(z)
		return x.NoCC and s.HasTalent(120679--[[Dire Beast]])
	end,
	EnemyTargetNeeded = true,
}

a.spells["A Murder of Crows with The Beast Within"] = {
	ID = 120657--[[A Murder of Crows]],
	RequireBuff = 34692--[[The Beast Within]],
	RequireBuffUnit = "player",
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Kill Command"] = {
	ID = 34026--[[Kill Command]],
	Check = function(z)
		return x.PetNoCC
	end,
	EnemyTargetNeeded = true,
	NoRangeCheck = true,
}

a.spells["Arcane Shot with Thrill of the Hunt"] = {
	ID = 3044--[[Arcane Shot]],
	RequireBuff = 34720--[[Thrill of the Hunt]],
	RequireBuffUnit = "player",
	Check = function(z)
		return x.NoCC and not a.PreSteadyFocus()
	end,
	EnemyTargetNeeded = true,
}

a.spells["Arcane Shot as focus dump when not hasted"] = {
	ID = 3044--[[Arcane Shot]],
	Check = function(z)
		return x.NoCC and a.canStillCast(53209--[[Chimera Shot]], 3044--[[Arcane Shot]]) and not s.Buff(a.HasteCDs, "player") and not a.PreSteadyFocus()
	end,
	EnemyTargetNeeded = true,
}

a.spells["Arcane Shot if focus >= 69 or The Beast Within up"] = {
	ID = 3044--[[Arcane Shot]],
	Check = function(z)
		return x.NoCC and ((s.Power("player", SPELL_POWER_FOCUS) >= 69) or s.Buff(34692--[[The Beast Within]],"player"))
	end,
	EnemyTargetNeeded = true,
}

a.spells["Arcane Shot if focus >= 67"] = {
	ID = 3044--[[Arcane Shot]],
	Check = function(z)
		return x.NoCC and s.Power("player", SPELL_POWER_FOCUS) >= 67
	end,
	EnemyTargetNeeded = true,
}

a.spells["Steady Shot with Steady Focus < 3"] = {
	ID = 56641--[[Steady Shot]],
	Check = function(z)
		return x.NoCC and a.PreSteadyFocus() and s.MyBuffDuration(53224--[[Steady Focus]], "player") < 3
	end,
	EnemyTargetNeeded = true,
}

a.spells["Steady Shot"] = {
	ID = 56641--[[Steady Shot]],
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Cobra Shot"] = {
	ID = 77767--[[Cobra Shot]],
	Check = function(z)
		return x.NoCC
	end,
	EnemyTargetNeeded = true,
}

a.spells["Rapid Fire without Haste buffs and without The Beast Within"] = {
	ID = 3045--[[Rapid Fire]],
	MyBuff = 3045--[[Rapid Fire]],
	BuffUnit = "player",
	Check = function(z)
		return x.ShouldTemporaryBuff and not s.Buff(a.HasteCDs, "player") and not s.Buff(34692--[[The Beast Within]], "player")
	end,
	Continue = true,
	NoGCD = true,
	FlashColor = "Yellow",
}

a.spells["Rapid Fire"] = {
	ID = 3045--[[Rapid Fire]],
	MyBuff = 3045--[[Rapid Fire]],
	BuffUnit = "player",
	Check = function(z)
		return x.ShouldTemporaryBuff
	end,
	Continue = true,
	NoGCD = true,
	FlashColor = "Yellow",
}

a.spells["Rapid Fire without Haste buffs or TTD <= 30"] = {
	ID = 3045--[[Rapid Fire]],
	MyBuff = 3045--[[Rapid Fire]],
	BuffUnit = "player",
	Check = function(z)
		return x.ShouldTemporaryBuff and (not s.Buff(a.HasteCDs, "player") or (s.TimeToDie() <= 30))
	end,
	Continue = true,
	NoGCD = true,
	FlashColor = "Yellow",
}

a.spells["Readiness after Rapid Fire"] = {
	ID = 23989--[[Readiness]],
	Check = function(z)
		return x.NoCC and s.SpellCooldown(3045--[[Rapid Fire]]) > s.GlobalCooldown()
	end,
	NoGCD = true,
}

a.spells["Aimed Shot with Master Marksman"] = {
	ID = 19434--[[Aimed Shot]],
	RequireMyBuff = 82926--[[Fire!]],
	RequireBuffUnit = "player",
	NotWhileMoving = false,
	Check = function(z)
		return x.NoCC and not a.PreSteadyFocus()
	end,
	EnemyTargetNeeded = true,
}

a.spells["Aimed Shot over 90% HP"] = {
	ID = 19434--[[Aimed Shot]],
	NotWhileMoving = true,
	Check = function(z)
		return x.NoCC and s.Health("target") > 0 and s.HealthPercent("target") > 90 and not a.PreSteadyFocus()
	end,
	EnemyTargetNeeded = true,
}

a.spells["Aimed Shot when hasted"] = {
	ID = 19434--[[Aimed Shot]],
	RequireBuff = a.HasteCDs,
	RequireBuffUnit = "player",
	NotWhileMoving = true,
	Check = function(z)
		return x.NoCC and not a.PreSteadyFocus()
	end,
	EnemyTargetNeeded = true,
}

a.spells["Call Pet"] = {
	ID = 9--[[Call Pet]],
	Override = function(z)
		if not x.PetAlive and x.ShouldPermanentBuff then
			s.Flash({
				9--[[Call Pet]],
				883--[[Call Pet 1]],
				83242--[[Call Pet 2]],
				83243--[[Call Pet 3]],
				83244--[[Call Pet 4]],
				83245--[[Call Pet 5]],
				982--[[Revive Pet]]})
		end
	end,
}

a.spells["Mend Pet"] = {
	ID = 136--[[Mend Pet]],
	Buff = 136--[[Mend Pet]],
	BuffUnit = "pet",
	EarlyRefresh = 1,
	Check = function(z)
		return x.ShouldTemporaryBuff and x.PetAlive and s.HealthPercent("pet") < 80
	end,
}

a.spells["Explosive Shot"] = {
	ID = 53301--[[Explosive Shot]],
	MyDebuffID = 53301--[[Explosive Shot]],
	EarlyRefresh = 1.9,
	Check = function(z)
		return x.NoCC and not s.SpellOrAuraDelay(53301--[[Explosive Shot]], "target", true)
	end,
	EnemyTargetNeeded = true,
}

a.spells["Black Arrow"] = {
	ID = 3674--[[Black Arrow]],
	MyDebuffID = 3674--[[Black Arrow]],
	Check = function(z)
		return x.NoCC
	end,
	Run = function(z)
		if s.TimeToDie() >= 8 then
			z.Continue = false
			z.FlashColor = "White"
		else
			z.Continue = true
			z.FlashColor = "Pink"
		end
	end,
	EnemyTargetNeeded = true,
}

a.spells["Disengage when movement impaired"] = {
	ID = 781--[[Disengage]],
	Check = function(z)
		return s.MovementImpaired("player") and s.HasTalent(109215--[[Posthaste]])
	end,
	FlashCOlor = "Yellow",
	NoGCD = true,
}

a.spells["Silencing Shot"] = {
	ID = 34490--[[Silencing Shot]],
	Debuff = 34490--[[Silencing Shot]],
	EnemyTargetNeeded = true,
	Interrupt = true,
	NoGCD = true,
	FlashColor = "Aqua",
}
